
#################################################
#################################################
##          COPYRIGHT - DATA SOCIETY           ##
#################################################
#################################################

################################
##    INTRO TO TEXT MINING    ##
################################

## NOTE: To run code blocks (separated by #%%), select somewhere inside the block 
##       and press ctrl + enter for Windows or Linux, 
##       and command + enter for Macs

#==================================================================================
#%% 
# Slide 15: Importing nltk 

# Import nltk as follows:
import nltk
nltk.download()

#==================================================================================
#%% 
# Slide 16: Importing text data

# Import text excerpts to use for this session:
from nltk.book import *

# You can pull out specific texts by referencing the variable name
text1
# <Text: Moby Dick by Herman Melville 1851>

# You can use texts() to look up all the text names
texts()

#==================================================================================
#%% 
# Slide 17: Searching text

'''
Here, we're telling Python the text we want to use,
the view we want, and which word to pull out.
'''

# Let's pull the word 'monstrous' from the Moby Dick text.
text1.concordance("monstrous")

#==================================================================================
#%% 
# Slide 18: Finding similar words in context

# Let's find words that appear similarly to "monstrous" in "Moby Dick".
text1.similar("monstrous")

# What words appear near "monstrous" 
# in Jane Austen's text?
text2.similar("monstrous")

#==================================================================================
#%% 
# Slide 19: Finding shared context

# Let's find the shared context between "monstrous" and "very" in 
# Jane Austen's text.
text2.common_contexts("monstrous", "very")

#==================================================================================
#%% 
# Slide 21: Plotting word dispersion

# In order to produce the visualization, make sure that matplotlib 
# and numpy are installed.
import matplotlib as ml
import numpy as np

# What is text4?
text4

# <Text: Inaugural Address Corpus>

'''
Here, we're telling Python the text we want to use,
the plot we want, and the words we want to plot.
'''

text4.dispersion_plot(["citizens", "democracy", "freedom", "duties", "America"])

#==================================================================================
#%% 
# Slide 23: Investigating vocabulary

# What is text3?
text3

# <Text: The Book of Genesis>


# Use len() to find the length of the text
len(text3)

# 44764

# This means there are 44,764 words and punctuation symbols, or "tokens", in this text. 

#==================================================================================
#%% 
# Slide 24: Investigating vocabulary

# If we want to find the distinct words and symbols, we can use set("text")
set(text3)

# But this is not very organized - if we wrap that in sorted(), the words
# and symbols will appear alphabetically
sorted(set(text3))

# We can find out the number of unique tokens with len()
len(set(text3))

# 2789

#==================================================================================
#%% 
# Slide 25: Investigating vocabulary

# How diverse is our text? We can figure it out by dividing the number of 
# unique words by the total number of words.

text6

# <Text: Monty Python and the Holy Grail>

lex_diversity = len(set(text6)) / len(text6)

# Now we can print the answer – we are printing it as a percentage by 
# multiplying it by 100 and adding a % sign at the end.
print("The lexical diversity is: ", lex_diversity * 100, "%")

#==================================================================================
#%% 
# Slide 26: Investigating vocabulary

# Let's look up a particular word in the text
text6.count("father")
# 7

# What is the percentage of 'the' in the text?
text6.count('the') / len(text6) * 100

# 1.76%

#==================================================================================
#%% 
# Slide 27: Investigating vocabulary

# If we want to calculate the lexical diversity of any text, we can create a 
# function for it!

def lex_divers(text):
     lex_diversity = len(set(text)) / len(text)
     return print("The lexical diversity is: ", 
                  lex_diversity * 100, "%")

# Here we'll add round() to our calculation so the number looks cleaner. The
# 2 after the comma is the number of decimal places we want to round to.      
def percent(text, word):
    text_count = round((text.count(word) / len(text) * 100), 2)
    return print("The word, ", word, ", appears in ", text, text_count, "% of the text")


# Let's test out our functions!
lex_divers(text9)    
percent(text5, "the")

#==================================================================================
#%% 
# Slide 28: Working with words in lists

# We can create lists with the [ ] syntax
sent1 = ['Call', 'me', 'Ishmael', '.']

# Let's check the length and lexical diversity.
len(sent1)
lex_divers(sent1)

# You can look at some more lists that were imported by typing in sent1 through sent9.
sent5

'''
['I',
 'have',
 'a',
 'problem',
 'with',
 'people',
 'PMing',
 'me',
 'to',
 'lol',
 'JOIN']
'''

#==================================================================================
#%% 
# Slide 26: Combining words in lists

# We can add two lists together with +
['Monty', 'Python'] + ['and', 'the', 'Holy', 'Grail']

# Combine variables that represent lists
sent1 + sent4

# Append lists with append()
sent1.append("No")
sent1

# ['Call', 'me', 'Ishmael', '.', 'No']

#==================================================================================
#%% 
# Slide 30: Indexing words

# We're pulling out the 563rd term from text4. 
text4[563]

# 'revolution'

# We're using the index() syntax to return the position of the term
# we highlighted from text4.
text4.index('revolution')
# 563

# Pull out a specific range of terms
text6[1501:1525]

# Remember! Python is a zero-index language, meaning that the first word has
# an index of zero. 

#==================================================================================
#%% 
# Slide 31: Indexing words

# If you want all the words up to a certain point, omit the first number.
text2[:7]
# ['[', 'Sense', 'and', 'Sensibility', 'by', 'Jane', 'Austen']

# If you want all the words after a certain point, omit the second number.
text2[141525:]

#==================================================================================
#%% 
# Slide 32: Sorting words

my_sent = ['Bravely', 'bold', 'Sir', 'Robin', ',', 'rode',
           'forth', 'from', 'Camelot', '.']

# Python sorts words by punctuation first, then uppercase, then lowercase.
phrase = my_sent[2:5]
word_sort = sorted(phrase)
word_sort

#==================================================================================
#%% 
# Slide 33: Working with strings

name = 'Monty'
name[0]
name[:3]

# Operations with strings
name * 2
name + '!'

# Join two words together with join()
' '.join(['Monty', 'Python'])

# Separate words with split()
'Monty Python'.split()

#==================================================================================
#%% 
# Slide 34: Frequency distribution

# We can use FreqDist() to determine the frequency distribution of text1.

fdist1 = FreqDist(text1) 
print(fdist1) 
# <FreqDist with 19317 samples and 260819 outcomes>

# What are the 50 most common words in Moby Dick?
fdist1.most_common(50)

# [(',', 18713), ('the', 13721), ('.', 6862), ('of', 6536), ('and', 6024),
# ('a', 4569), ('to', 4542), (';', 4072), ('in', 3916), ('that', 2982),
# ("'", 2684), ('-', 2552), ('his', 2459), ('it', 2209), ('I', 2124),
# ('s', 1739), ('is', 1695), ('he', 1661), ('with', 1659), ('was', 1632),
# ('as', 1620), ('"', 1478), ('all', 1462), ('for', 1414), ('this', 1280),
# ('!', 1269), ('at', 1231), ('by', 1137), ('but', 1113), ('not', 1103),
# ('--', 1070), ('him', 1058), ('from', 1052), ('be', 1030), ('on', 1005),
# ('so', 918), ('whale', 906), ('one', 889), ('you', 841), ('had', 767),
# ('have', 760), ('there', 715), ('But', 705), ('or', 697), ('were', 680),
# ('now', 646), ('which', 640), ('?', 637), ('me', 627), ('like', 624)]

# We can pull out the frequency of a particular word.
fdist1['whale']
# 906

#==================================================================================
#%% 
# Slide 35: Frequency distribution plot

# What does the distribution of words look like?
fdist1.plot(50, cumulative = False)

# Let's create a cumulative plot of the fifty most popular words
fdist1.plot(50, cumulative = True)

# These 50 words comprise almost 50% of the book!

# We can look for hapaxes as well - these are words that only occur once.
fdist1.hapaxes()

#==================================================================================
#%% 
# Slide 37: Subsetting words

# What if we want to find words that contain particular constraints?
# Let's use a for loop to pull out all words with a length greater than 15.
 	
V = set(text1)

long_words = [w for w in V if len(w) > 15]
sorted(long_words)

# ['CIRCUMNAVIGATION', 'Physiognomically', 'apprehensiveness', 'cannibalistically',
# 'characteristically', 'circumnavigating', 'circumnavigation', 'circumnavigations',
# 'comprehensiveness', 'hermaphroditical', 'indiscriminately', 'indispensableness',
# 'irresistibleness', 'physiognomically', 'preternaturalness', 'responsibilities',
# 'simultaneousness', 'subterraneousness', 'supernaturalness', 'superstitiousness',
# 'uncomfortableness', 'uncompromisedness', 'undiscriminating', 'uninterpenetratingly']

#==================================================================================
#%% 
# Slide 38: Subsetting words

# What if we want to find long words that are frequently occurring?

fdist2 = FreqDist(text2)

sorted(w for w in set(text2) if len(w) > 7 and fdist2[w] > 7)

#==================================================================================
#%% 
# Slide 40: Finding bigrams

# We can use the bigrams() function to extract word pairs that occur together unusually
# often.
list(nltk.bigrams(['more', 'is', 'said', 'than', 'done']))

# [('more', 'is'), ('is', 'said'), ('said', 'than'), ('than', 'done')]

'''
If you omitted list() above, and just typed bigrams(['more', ...]), you would 
have seen output of the form <generator object bigrams at 0x10fb8b3a8>. 
This is Python's way of saying that it is ready to compute a sequence of items, 
in this case, bigrams. For now, you just need to know to tell Python to convert 
it into a list, using  list().
'''
#==================================================================================
#%% 
# Slide 41: Finding collocations

# Collocations are sequences of words that occur unusually often and contain rare words.
# What are the most common word occurrences in an Inaugural Address?
text4.collocations()

# What about a corpus that contains personal ads?
text8.collocations()

#==================================================================================
#%% 
# Slide 42: Counting word lengths

# Return the lengths of each word in text1. 
[len(w) for w in text1] 
# [1, 4, 4, 2, 6, 8, 4, 1, 9, 1, 1, 8, 2, 1, 4, 11, 5, 2, 1, 7, 6, 1, 3, 4, 5, 2, ...]

# Now, wrap that code in FreqDist to see the frequency distribution of word lengths.
fdist = FreqDist(len(w) for w in text1)
print(fdist)

# <FreqDist with 19 samples and 260819 outcomes>

fdist

# Plot the top 5 frequently occurring word lengths
fdist.plot(5)

#==================================================================================
#%% 
# Slide 44: Counting word lengths

# We're using the most_common() syntax to pull out the most 
# common occurrences of fdist.
fdist.most_common()

# [(3, 50223), (1, 47933), (4, 42345), (2, 38513), (5, 26597), (6, 17111), (7, 14399),
# (8, 9966), (9, 6428), (10, 3528), (11, 1873), (12, 1053), (13, 567), (14, 177),
# (15, 70), (16, 22), (17, 12), (18, 1), (20, 1)]

# The max() syntax identifies the term that has the 
# highest occurrence.
fdist.max()

# 3

# Here, we're asking how many occurrences there were for the term '3'.
fdist[3]

# 50223

# What is the percent of 3-letter words in the text?
fdist.freq(3)

# 0.19255882431878046

#==================================================================================
#%% 
# Slide 47: Word comparison operators

# Find all the words in text1 that end in 'ableness'.
sorted(w for w in set(text1) if w.endswith('ableness'))

# ['comfortableness', 'honourableness', 'immutableness', 'indispensableness', ...]

# Find all the words in text4 that contain 'gnt'. 
sorted(term for term in set(text4) if 'gnt' in term)

# ['Sovereignty', 'sovereignties', 'sovereignty']

# Find all the words in text6 if words are capitalized
sorted(item for item in set(text6) if item.istitle())

# ['A', 'Aaaaaaaaah', 'Aaaaaaaah', 'Aaaaaah', 'Aaaah', 'Aaaaugh', 'Aaagh', ...]

# Find all the terms in sent7 that are numbers.
sorted(item for item in set(sent7) if item.isdigit())
# ['29', '61']

#==================================================================================
#%% 
# Slide 49: More complex operators

# Find all the words in text7 that contain '-' and 'index'. 	
sorted(w for w in set(text7) if '-' in w and 'index' in w)

# Find all the words in text4 that are capitalized with a 
# length greater than 10. 
sorted(wd for wd in set(text3) if wd.istitle() and len(wd) > 10)

# Find all the words in text7 if words are uppercase.
sorted(w for w in set(sent7) if not w.islower())

# Find all the terms in text2 that have either 'cie' or 'cei'.
sorted(t for t in set(text2) if 'cie' in t or 'cei' in t)

#==================================================================================
#%% 
# Slide 51: Dealing with word cases

# Eliminate double counting of the same words that have different cases
# (i.e. 'This' vs 'this')
len(set(word.lower() for word in text1 if word.isalpha()))
# 16948

#==================================================================================
#%% 
# Slide 52: Looping with conditions

sent1 = ['Call', 'me', 'Ishmael', '.']

# Let's combine the for and if statements to take an action if a 
# condition is met.
for word in sent1:
    if word.endswith('l'):
        print(word)

#==================================================================================
#%% 

# Slide 53: Looping with conditions

for token in sent1:
    if token.islower():
        print(token, 'is a lowercase word')
    elif token.istitle():
        print(token, 'is a titlecase word')
    else:
        print(token, 'is punctuation')

#==================================================================================
#%% 
# Slide 55: Importing larger texts

# We can access the Gutenberg texts with the following code:
nltk.corpus.gutenberg.fileids()

# Let's pick out the first of these texts — Emma by Jane Austen — and give 
# it a short name, emma, then find out how many words it contains:
emma = nltk.corpus.gutenberg.words('austen-emma.txt')
len(emma)

# 192427

#==================================================================================
#%% 
# Slide 56: Importing larger texts

# First, we can assign the text to a a shorter variable, 'emma'.
emma = nltk.Text(nltk.corpus.gutenberg.words('austen-emma.txt'))

#==================================================================================
#%% 
# Slide 57: Importing larger texts

# Now, we can use concordance to find the words around 'surprize'
emma.concordance("surprize")

#==================================================================================
#%% 
# Slide 58: Importing larger texts

'''
Instead of having to assign a text with that cumbersome syntax, Python 
provides another way for us to import the text
'''
# Import the gutenberg texts from nltk.corpus
from nltk.corpus import gutenberg

# Look at the names of the files
gutenberg.fileids()

# ['austen-emma.txt', 'austen-persuasion.txt', 'austen-sense.txt', ...]

# Set the variable name 'emma' equal to the text "Emma" by Jane Austen
emma = gutenberg.words('austen-emma.txt')

#==================================================================================
#%% 
# Slide 59: Automating information


for fileid in gutenberg.fileids():
    # The length of the raw file (number of characters)
    num_chars = len(gutenberg.raw(fileid)) 
    # The number of words in the file
    num_words = len(gutenberg.words(fileid))
    # The number of sentences in the file
    num_sents = len(gutenberg.sents(fileid))
    # The number of unique words in the file
    num_vocab = len(set(w.lower() for w in gutenberg.words(fileid)))
    print(fileid, num_chars, num_words, num_sents, num_vocab)
    
#==================================================================================
#%% 
# Slide 60: Automating information
    
# What if we want to look at additional metrics?    
for fileid in gutenberg.fileids():
    # The length of the raw file (number of characters)
    num_chars = len(gutenberg.raw(fileid)) 
    # The number of words in the file
    num_words = len(gutenberg.words(fileid))
    # The number of sentences in the file
    num_sents = len(gutenberg.sents(fileid))
    # The number of unique words in the file
    num_vocab = len(set(w.lower() for w in gutenberg.words(fileid)))
    # Print the metrics we want to see – the number of characters per word, the     
    # number of words per sentence, and the diversity of vocabulary.
    print(round(num_chars/num_words), round(num_words/num_sents), 
          round(num_vocab/num_words * 100, 2), "% ", fileid)

#==================================================================================
#%%
# Slide 61: Segmenting texts

# We can divide up each text into sentences with sents() 	
macbeth_sentences = gutenberg.sents('shakespeare-macbeth.txt')
macbeth_sentences

# [['[', 'The', 'Tragedie', 'of', 'Macbeth', 'by', 'William', 'Shakespeare',
# '1603', ']'], ['Actus', 'Primus', '.'], ...]

macbeth_sentences[1116]

# ['Double', ',', 'double', ',', 'toile', 'and', 'trouble', ';',
# 'Fire', 'burne', ',', 'and', 'Cauldron', 'bubble']

longest_len = max(len(s) for s in macbeth_sentences)
[s for s in macbeth_sentences if len(s) == longest_len]

# [['Doubtfull', 'it', 'stood', ',', 'As', 'two', 'spent', 'Swimmers', ',', 'that',
# 'doe', 'cling', 'together', ',', 'And', 'choake', 'their', 'Art', ':', 'The',
# 'mercilesse', 'Macdonwald', ...]]

# Most NLTK corpus readers include a variety of access methods apart from words(), raw(), and sents(). 

#==================================================================================
#%% 
# Slide 64: The Brown corpus


# First, import the Brown corpus and view the categories 	
from nltk.corpus import brown
brown.categories()

# ['adventure', 'belles_lettres', 'editorial', 'fiction', 'government', 'hobbies',
# 'humor', 'learned', 'lore', 'mystery', 'news', 'religion', 'reviews', 'romance',
# 'science_fiction']

# Which words are in the 'news' category?
brown.words(categories='news')

# ['The', 'Fulton', 'County', 'Grand', 'Jury', 'said', ...]

# Which words are in the file id 'cg22'?
brown.words(fileids=['cg22'])

# ['Does', 'our', 'society', 'have', 'a', 'runaway', ',', ...]

# Which sentences are in the categories 'news', 'editorial', and 'reviews'?
brown.sents(categories=['news', 'editorial', 'reviews'])
# [['The', 'Fulton', 'County'...], ['The', 'jury', 'further'...], ...]

#==================================================================================
#%% 
# Slide 65: The Brown corpus


# First, import the Brown corpus
from nltk.corpus import brown

# Define the 'news_text' variable as the words in the 'news' category
news_text = brown.words(categories='news')

# Find the frequency distribution for the words 
# Make sure to use the lower() syntax so that we're not double counting words
fdist = nltk.FreqDist(w.lower() for w in news_text)

# Define the words we want to look at, and then print the distribution
modals = ['can', 'could', 'may', 'might', 'must', 'will']
for m in modals:
    print(m + ':', fdist[m], end=' ')

# can: 94 could: 87 may: 93 might: 38 must: 53 will: 389
    
'''
Note: We need to include end=' ' in order for the print function to put 
its output on a single line.
'''
    
#==================================================================================
#%% 
# Slide 66: The Brown corpus

'''
Next, we need to obtain counts for each genre of interest. We'll use NLTK's 
support for conditional frequency distributions. 
These are presented systematically later in the lecture, where we also unpick 
the following code line by line. 

For the moment, you can ignore the details and just concentrate on the output.

'''
 	
cfd = nltk.ConditionalFreqDist(
          (genre, word)
          for genre in brown.categories()
          for word in brown.words(categories=genre))
genres = ['news', 'religion', 'hobbies', 'science_fiction', 'romance', 'humor']
modals = ['can', 'could', 'may', 'might', 'must', 'will']

#==================================================================================
#%% 
# Slide 67: The Brown corpus

cfd.tabulate(conditions = genres, samples = modals)
#                  can could  may might must will
#            news   93   86   66   38   50  389
#        religion   82   59   78   12   54   71
#         hobbies  268   58  131   22   83  264
# science_fiction   16   49    4   12    8   16
#         romance   74  193   11   51   45   43
#           humor   16   30    8    8    9   13

'''
What are some facts that you observe about these numbers?
'''

#==================================================================================
#%% 
# Slide 68: The Reuters corpus

# Import the Reuters corpus
from nltk.corpus import reuters
reuters.fileids()

# ['test/14826', 'test/14828', 'test/14829', 'test/14832', ...]

# Look at the file ids and categories
reuters.categories()

# ['acq', 'alum', 'barley', 'bop', 'carcass', 'castor-oil', 'cocoa',
# 'coconut', 'coconut-oil', 'coffee', 'copper', 'copra-cake', 'corn',
# 'cotton', 'cotton-oil', 'cpi', 'cpu', 'crude', 'dfl', 'dlr', ...]

#==================================================================================
#%% 
# Slide 69: The Reuters corpus

# What do some of the texts show?	
reuters.categories('training/9865')

#['barley', 'corn', 'grain', 'wheat']

reuters.categories(['training/9865', 'training/9880'])

# ['barley', 'corn', 'grain', 'money-fx', 'wheat']

reuters.fileids('barley')

# ['test/15618', 'test/15649', 'test/15676', 'test/15728', 'test/15871', ...]

reuters.fileids(['barley', 'corn'])

# ['test/14832', 'test/14858', 'test/15033', 'test/15043', 'test/15106',
# 'test/15287', 'test/15341', 'test/15618', 'test/15648', 'test/15649', ...]

#==================================================================================
#%% 
# Slide 70: The Reuters corpus

# How do we pull out specific words? 

reuters.words('training/9865')[:14]

# ['FRENCH', 'FREE', 'MARKET', 'CEREAL', 'EXPORT', 'BIDS', 'DETAILED', 
# 'French', 'operators', 'have', 'requested', 'licences', 'to', 'export']

reuters.words(['training/9865', 'training/9880'])

# ['FRENCH', 'FREE', 'MARKET', 'CEREAL', 'EXPORT', ...]

reuters.words(categories='barley')

# ['FRENCH', 'FREE', 'MARKET', 'CEREAL', 'EXPORT', ...]

reuters.words(categories=['barley', 'corn'])

# ['THAI', 'TRADE', 'DEFICIT', 'WIDENS', 'IN', 'FIRST', ...]

#==================================================================================
#%% 
# Slide 71: The Inaugural Address corpus


# Import the Inaugural Address corpus
from nltk.corpus import inaugural
inaugural.fileids()

# ['1789-Washington.txt', '1793-Washington.txt', '1797-Adams.txt', ...]

# We can extract the first four characters in each text to view the year
# for each address. We can pull it out with a for loop. 
[fileid[:4] for fileid in inaugural.fileids()]

# ['1789', '1793', '1797', '1801', '1805', '1809', '1813', '1817', '1821', ...]


#==================================================================================
#%% 
# Slide 72: The Inaugural Address corpus

# Let's plot the word usage over each text – we're converting the 
# text to lowercase, then check if the target starts with 'america' or
# 'citizen' with startswith() to include words like "American" and "citizens".
cfd = nltk.ConditionalFreqDist(
          (target, fileid[:4])
          for fileid in inaugural.fileids()
          for w in inaugural.words(fileid)
          for target in ['america', 'citizen']
          if w.lower().startswith(target))

#==================================================================================
#%% 
# Slide 73: The Inaugural Address corpus

# Let's plot the distribution
cfd.plot()

#==================================================================================
#%% 
# Slide 74: Text mining in different languages


# NLTK comes with corpora for many languages, though in some cases you will 
# need to learn how to manipulate character encodings in Python before using these corpora.

# Make sure that the udhr and udhr2 corpora are downloaded by clicking on the 
# 'corpora' tab, scrolling down to the names, highlighting them, and clicking "download"
nltk.download()

#==================================================================================
#%% 
# Slide 75: Text mining in different languages

# Spanish 
nltk.corpus.cess_esp.words()

# Portuguese
nltk.corpus.floresta.words()

# Hindi
nltk.corpus.indian.words('hindi.pos')

# The udhr corpus contains the Universal Declaration
# of Human Rights in over 300 languages
nltk.corpus.udhr.fileids()


#==================================================================================
#%% 
# Slide 76: Text mining in different languages
 	
# Make sure udhr is imported
from nltk.corpus import udhr

# Define the languages you want to use
languages = ['Chickasaw', 'English', 'German_Deutsch',
     'Greenlandic_Inuktikut', 'Icelandic_Yslenska', 'Norwegian']

# Set the conditional frequency distribution for word length
cfd = nltk.ConditionalFreqDist(
	(lang, len(word))
    for lang in languages
    for word in udhr.words(lang + '-Latin1'))


#==================================================================================
#%% 
# Slide 77: Text mining in different languages

# Let's plot the distribution
cfd.plot(cumulative = True)


#==================================================================================
#%% 
# Slide 79: Loading your own corpus

# Import the PlaintextCorpusReader
from nltk.corpus import PlaintextCorpusReader

# Whatever the file location, set it to 'corpus_root
corpus_root = '/usr/share/dict' 

# The second parameter can be a list of file ids or patterns that match
# all file ids
wordlists = PlaintextCorpusReader(corpus_root, '.*') 
wordlists.fileids()

# ['README', 'connectives', 'propernames', 'web2', 'web2a', 'words']

wordlists.words('connectives')
# ['the', 'of', 'and', 'to', 'a', 'in', 'that', 'is', ...]


#==================================================================================
#%% 
# Slide 82: Counting words by category


# Here is how we have used ConditionalFreqDist() in the past:
from nltk.corpus import brown
cfd = nltk.ConditionalFreqDist(
          (genre, word)
          for genre in brown.categories()
          for word in brown.words(categories=genre))

#==================================================================================
#%% 
# Slide 83: Counting words by category

# Let's take two genres, news and romance, and loop over every word to 
# produce pairs of the genre and the word:

genre_word = [(genre, word) 
              for genre in ['news', 'romance'] 
              for word in brown.words(categories = genre)] 

len(genre_word)
# 170576


# The first pairs will have the genre 'news', and then the last pairs will
# have the genre 'romance' because that's the order we put the genres in.

# The first four pairs
genre_word[:4]

# ('news', 'The'), ('news', 'Fulton'), ('news', 'County'), ('news', 'Grand')] # [_start-genre]

# The last four pairs
genre_word[-4:]
# [('romance', 'afraid'), ('romance', 'not'), ('romance', "''"), ('romance', '.')] # [_end-genre]

#==================================================================================
#%% 
# Slide 84: Counting words by category

# Now we can use ConditionalFreqDist() on the list of pairs we created
cfd = nltk.ConditionalFreqDist(genre_word)

# We can type in the variable to inspect it and check the conditions
cfd
cfd.conditions()


# We can also check the frequency distributions for the genres
print(cfd['news'])

# <FreqDist with 14394 samples and 100554 outcomes>

print(cfd['romance'])

# <FreqDist with 8452 samples and 70022 outcomes>

cfd['romance'].most_common(10)

cfd['romance']['that']
# 583

#==================================================================================
#%% 
# Slide 85: Plotting distributions

# Here, we're combining the previous code into one block. This is 
# similar to the Inaugural Address plot. We create the conditional frequency
# distribution and counted the occurrences for each word.
from nltk.corpus import inaugural
cfd = nltk.ConditionalFreqDist(
          (target, fileid[:4]) 
          for fileid in inaugural.fileids()
          for w in inaugural.words(fileid)
          for target in ['unite', 'divide'] 
          if w.lower().startswith(target))

cfd.plot()

#==================================================================================
#%% 
# Slide 86: Tabulating occurrences

from nltk.corpus import udhr
languages = ['Chickasaw', 'English', 'German_Deutsch',
     'Greenlandic_Inuktikut', 'Icelandic_Yslenska', 'Norwegian']

# Set the conditional frequency distribution for word length
cfd = nltk.ConditionalFreqDist(
          (lang, len(word))
          for lang in languages
          for word in udhr.words(lang + '-Latin1'))

#==================================================================================
#%% 
# Slide 87: Tabulating occurrences

cfd.tabulate(conditions=['English', 'German_Deutsch'],
             samples = range(10), cumulative = False)








